<?php
/**
* Main controller file for WebMoney WMI Component 
*
* @package com_wmi
* @version 1.5.0
* @copyright Copyright (C) 2005 Open Source Matters. All rights reserved.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
*/

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Access restricted');

jimport('joomla.application.component.controller');

/**
 * WMI Component Controller
 *
 * @package		WMI
 */
class WMIController extends JController
{
	/**
	 * Error string
	 *
	 * @var string
	 */
	var $_errmsg = "";

	/**
	 * Message subject
	 *
	 * @var string
	 */
	var $_subject = "WMI notification";

	/**
	 * Method to display the view
	 *
	 * @access	public
	 */
	function display()
	{
		parent::display();
	}

	/**
	 * Method to display the Success view
	 *
	 * @access	public
	 */
	function success()
	{
                JRequest::setVar('view', 'success');
		parent::display();
	}

	/**
	 * Method to display failure view
	 *
	 * @access	public
	 */
	function failure()
	{
                JRequest::setVar('view', 'failure');
		parent::display();
	}

	/**
	 * Method to handle response from the WMI server
	 *
	 * @access	public
	 */
	function notify()
	{
		// Get parameters from the database
		$params = &JComponentHelper::getParams( 'com_wmi' );	
		
		// Check the data returned from WMI servers
		WMIController::_check_params();

		// Do prerequest handling, checks
		if ( JRequest::getVar('LMI_PREREQUEST', '0', 'POST') == 1) {

			// See if all checks are Okay
			if( strlen( $this->_errmsg ) > 0) {
				// Notify by email if necessary
				$this->_subject = "WMI prerequest error";
			} else
				echo "YES";	// Print "YES" in order to proceed with this payment
		} else { // Do post payment check, database update

			// Check the secret key if present
			if( strlen(JRequest::getVar('LMI_SECRET_KEY', '', 'POST')) > 0 && 
				JRequest::getVar('LMI_SECRET_KEY', '', 'POST') !== $params->get( 'secret_key'))
				$this->_errmsg .= "The secret key (LMI_SECRET_KEY) does not match to stored one.\n";

			// Concat string to hash
			$amount = JRequest::getVar('LMI_PAYMENT_AMOUNT', '0.00', 'POST');
			$invoice = JRequest::getVar('LMI_PAYMENT_NO', '', 'POST');

			$chkstring = $params->get( 'purse').$amount.$invoice.JRequest::getVar('LMI_MODE', '2', 'POST').
			JRequest::getVar('LMI_SYS_INVS_NO', '', 'POST').JRequest::getVar('LMI_SYS_TRANS_NO', '', 'POST').
			JRequest::getVar('LMI_SYS_TRANS_DATE', '19700101 00:00:00', 'POST').$params->get( 'secret_key').
			JRequest::getVar('LMI_PAYER_PURSE', '', 'POST').JRequest::getVar('LMI_PAYER_WM', '', 'POST');
		    	$md5sum = strtoupper(md5($chkstring));

			// Check the hash string
			if( JRequest::getVar('LMI_HASH', '', 'POST') !== $md5sum)
				$this->_errmsg .= "The hash string (LMI_HASH) does not match to calculated one (".$md5sum.").\n";

			// See if all checks are Okay
			if( strlen( $this->_errmsg ) > 0) {
				// Notify by email if necessary
				$this->_subject = "WMI request error";
			} else {
				// Insert transaction detail to the database
				$model = $this->getModel('wmi');
				if(!$model->insert()) {
					// Notify by email if necessary
					$this->_subject = "WMI database error";
				}
/*********************************************************
Real integration starts here. All validation cheks were already completed.
Data was added to the database for later processing.
The user has just made a payment. Enable him immediately.
Now you should do something about changing user access rights 
or giving him the link to download something or whatever.
It is hardly possible to unify the integration process.
If you find it difficult to proceed with integration
please do not hesitate to contact PaySysPro http://www.paysyspro.com.
We can do the integration work for you.
*************************************************************/
			}
		}

		// Notify by email if necessary
		if ( $params->get( 'email_notification') == 1)
			WMIController::_sendMail( $this->_subject, $this->_errmsg );

		// Do not do anything else, just exit.
		exit();
	}

	/**
	 * Method to check the params passed in the request
	 *
	 * @access	public
	 */
	function _check_params()
	{
		// Get parameters from the database
		$params = &JComponentHelper::getParams( 'com_wmi' );	

		// Check the test mode
		if ( JRequest::getVar('LMI_MODE', '2', 'POST') !== $params->get( 'test_mode'))
			$this->_errmsg .= "Test code (LMI_MODE) does not match with config variable (".$params->get( 'test_mode').").\n";

		// Check the merchant purse
		if( JRequest::getVar('LMI_PAYEE_PURSE', '', 'POST') != $params->get( 'purse'))
			$this->_errmsg .= "Wrong purse (LMI_PAYEE_PURSE). We have ".$params->get( 'purse')." in config.\n";

/*********************************************************
Here it is necessary to check if the transaction was originally sent by WMI servers 
by examining the LMI_HASH value and comparing it to the calculated one.
Or it is possible to check the secret key value from the request.
It is absolutely necessary to check if the LMI_MODE is equal to the config variable.
Payee purse should be validated aswell.
It is a good idea to check payment amount and invoice number.
Nothing to output at this stage. Just show blank screen.
To check these vars you should get the values from your database or your payment tracking system.
If the amount for the specified invoice do not match reject the payment.
LMI_PAYMENT_AMOUNT
LMI_PAYMENT_NO 
Read more about notification validation here:
https://merchant.wmtransfer.com/conf/guide.asp#validation
*************************************************************/
	}
	
	/**
	 * Method to send a debug notification
	 *
	 * @access	private
	 */
        function _sendMail( $title, $body)
        {
                jimport('joomla.environment.browser');
                global $mainframe;
                $mailfrom               = $mainframe->getCfg( 'mailfrom' );
                $fromname               = $mainframe->getCfg( 'fromname' );
		$uri 			= new JURI( JURI::root() );
		$browser		=& JBrowser::getInstance();
                $message = "The request of the WMI notification URL was made:\nFrom: ".$_SERVER['REMOTE_ADDR'].
                        "\nURL: ".$uri->toString().JRequest::getURI()."\nUserAgent: ".$browser->getAgentString()."\n\n".$body."\n\n";
		$data = JRequest::get( 'POST' );
		foreach ($data as $key => $value)
			// Insert all the records except for secret_key and submit
			if( $key != "submit" && $key != "secret_key")
				$message .= $key.": ".$value."\n";
		// Send message using Joomla mailer to the admin
                JUtility::sendMail($mailfrom, $fromname, $mailfrom, $title, $message);
	}
}
?>
